<?php

namespace FiloBlu\Refilo\Model\Exporter;

use Blackbird\ContentManager\Model\ResourceModel\Content\CollectionFactory;
use Blackbird\ContentManager\Model\ResourceModel\ContentType\CollectionFactory as ContentTypeCollectionFactory;
use FiloBlu\Refilo\Helper\ContentManager\Data as ContentManagerHelper;
use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Store\Model\StoreManagerInterface;

class ContentManager
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var Mongo
     */
    private $mongo;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var CollectionFactory
     */
    protected $contentCollectionFactory;
    /**
     * @var ContentTypeCollectionFactory
     */
    private $contentTypeCollectionFactory;

    protected $contentTypes;
    /**
     * @var ContentManagerHelper
     */
    private $contentManagerHelper;

    /**
     * Menu constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     * @param CollectionFactory $contentCollectionFactory
     * @param ContentTypeCollectionFactory $contentTypeCollectionFactory
     * @param ContentManagerHelper $contentManagerHelper
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager,
        CollectionFactory $contentCollectionFactory,
        ContentTypeCollectionFactory $contentTypeCollectionFactory,
        ContentManagerHelper $contentManagerHelper
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->scopeConfig = $scopeConfig;
        $this->mongo = $mongo;
        $this->storeManager = $storeManager;
        $this->contentCollectionFactory = $contentCollectionFactory;
        $this->contentTypeCollectionFactory = $contentTypeCollectionFactory;
        $this->contentManagerHelper = $contentManagerHelper;
    }

    /**
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function exportMasterData()
    {
        foreach ($this->storeManager->getStores() as $key => $value) {
            $this->mongo->deleteDocumentsByProperties(
                $this->scopeConfig->getValue('mongo/general/mongo_general_database'),
                "master_data_" . $value->getCode(),
                ['metacode' => 'master_data']
            );

            $bulkData = [];
            $result = $this->contentCollectionFactory->create()->addStoreFilter($value)->addAttributeToSelect('*');
            foreach ($result as $item) {
                $itemIdentifier = $this->contentManagerHelper->getIdentifierFromId($item->getData('ct_id'));
                $content = new \stdClass();

                $content->_id = $itemIdentifier . "_" . $item->getId() . "_" . $value->getData('code');
                $content->metacode = 'master_data';
                $content->contentmetacode = $itemIdentifier;
                $content->idx = $item->getId();
                $content->metadata = $this->getContentMetadata($item->getData('ct_id'), $item);

                $bulkData[] = $content;
            }

            try {
                $this->mongo->update(
                    '_id',
                    $bulkData,
                    'master_data_' . $value->getCode(),
                    true,
                    $this->scopeConfig->getValue('mongo/general/mongo_general_database')
                );
            } catch (\MongoDB\Driver\Exception\Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $ctId
     * @param $item
     * @return array
     */
    public function getContentMetadata($ctId, $item)
    {
        return array_merge($this->contentManagerHelper->getDataFromId($ctId), $item->getData());
    }
}
